@@ -99,7 +99,12 @@ def main():
             default=None)
     parser.add_argument('--n_events_combine', dest='n_events_combine', type=int,
         help='Integer to specify the number events from each file to pool into the final Infinity Flow object.',
-        default=None)
+        default=10000)
+    parser.add_argument('--require_n_events_combine', dest='require_n_events_combine', type=str,
+        help='Boolean to specify behavior if n_events_combine value is insufficient. \n'\
+            'If True, the program will stop if an FCS file does not have enough events. \n'\
+            'If False, n_events_combine will take the number of validation events.',
+        default="True")
     parser.add_argument('--n_final', dest='n_final', type=int,
         help='Integer to specify the final number of events to include for the final \n'\
             'Infinity Flow object. If n_final is 0, then all available events from \n'\
@@ -125,7 +130,7 @@ def main():
             'reduce the feature space prior to UMAP and clustering. This is suggested '\
             'to save computation time.',
         default="True")
-    parser.add_argument('--n_pc', dest='n_pc', type=str,
+    parser.add_argument('--n_pc', dest='n_pc', type=int,
         help='Integer to specify the number principal components to use for UMAP.',
         default=15)
     parser.add_argument('--n_pc_plot_qc', dest='n_pc_plot_qc', type=int,
@@ -165,6 +170,7 @@ def main():
         ratio_for_validation = args.ratio_for_validation
         separate_backbone_reference = parse_none_string(args.separate_backbone_reference)
         n_events_combine = parse_none_string(args.n_events_combine)
+        require_n_events_combine = parse_boolean_string(args.require_n_events_combine)
         n_final = args.n_final
         add_umap = parse_boolean_string(args.add_umap)
         find_clusters = parse_boolean_string(args.find_clusters)
@@ -215,6 +221,7 @@ def main():
         n_events_train=n_events_train, 
         n_events_validate=n_events_validate, 
         n_events_combine=n_events_combine, 
+        require_n_events_combine=require_n_events_combine,
         ratio_for_validation=ratio_for_validation,
         separate_backbone_reference=separate_backbone_reference,
         random_state=RANDOM_STATE, 
@@ -271,6 +278,7 @@ def main():
                 cores_to_use = cores_to_use, 
                 verbosity = VERBOSITY)
     else:
+        background_corrected_data = None
         timings_4 = {}
 
     # Use only the features from the backbone and prediction channels for
@@ -349,7 +357,8 @@ def main():
             v1="Plotting Leiden clusters over UMAP...")
         plot_leiden_clusters_over_umap(\
             sub_p_adata=sub_p_adata, 
-            output_paths=output_paths, 
+            file_path=os.path.join(output_paths["clustering"], 
+                "Leiden_Clusters_over_UMAP.png"), 
             verbosity=VERBOSITY)
         
 
@@ -360,7 +369,7 @@ def main():
         t_find_markers_start = time.time()
         markers_df, cell_assignments = find_markers_from_anndata(\
             sub_p_adata=sub_p_adata, 
-            output_paths=output_paths, 
+            output_dir=output_paths["clustering"], 
             groups_to_colors=sub_p_adata.uns['groups_to_color'], 
             verbosity=VERBOSITY)
         sub_p_adata.uns['markers_df'] = markers_df
